//   Copyright (c)  2016  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
//#include "CoCoA/assert.H"  // for using TEST_ASSERT
#include "CoCoA/error.H"
#include "CoCoA/RingFqLog.H"
#include "CoCoA/RingFqVec.H"
#include "CoCoA/IntOperations.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/RingHom.H"
#include "CoCoA/RingFp.H"


#include <iostream>
using std::cerr;
using std::endl;


// Handy macro for making assertions.
#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


namespace CoCoA
{

  // Put your code inside namespace CoCoA to avoid possibile
  // ambiguities with STL fns sharing names with CoCoALib fns.

  void program()
  {
    GlobalManager CoCoAFoundations;

    int p = 101;
    const int DegMinPoly = 3;
    const long CARD = SmallPower(p,DegMinPoly);

    ring FqVec = NewRingFqVec(p,DegMinPoly);

    TEST_ASSERT(characteristic(FqVec) == p);
    TEST_ASSERT(LogCardinality(FqVec) == DegMinPoly);
    TEST_ASSERT(IsQuotientRing(FqVec));
    TEST_ASSERT(IsPolyRing(BaseRing(FqVec)));
    TEST_ASSERT(IsRingFp(CoeffRing(BaseRing(FqVec))));

    ring FqLog = NewRingFqLog(p,DegMinPoly);

    TEST_ASSERT(characteristic(FqLog) == p);
    TEST_ASSERT(LogCardinality(FqLog) == DegMinPoly);
    TEST_ASSERT(IsQuotientRing(FqLog));
    TEST_ASSERT(IsPolyRing(BaseRing(FqVec)));
    TEST_ASSERT(IsRingFp(CoeffRing(BaseRing(FqVec))));

    {
      ring Fpx = BaseRing(FqVec);
      RingElem x = indet(Fpx,0);
      RingElem gen = QuotientingHom(FqVec)(x);
      gen = power(gen,29641);
      RingElem pwr = one(FqVec);
      for (int i=0; i < CARD-1; ++i)
      {
        TEST_ASSERT(i == 0 || !IsOne(pwr));
        pwr *= gen;
      }
      TEST_ASSERT(IsOne(pwr));
    }


    {
      ring FpxLog = BaseRing(FqLog);
      RingElem x = indet(FpxLog,0);
      RingElem gen = QuotientingHom(FqLog)(x);
      gen = power(gen,29641);
      RingElem pwr = one(FqLog);
      TEST_ASSERT(IsOne(pwr));
      for (int i=0; i < CARD-1; ++i)
      {
        TEST_ASSERT(i == 0 || !IsOne(pwr));
        pwr *= gen;
      }
      TEST_ASSERT(IsOne(pwr));
    }

  }

} // end of namespace CoCoA


// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}
